@@ -25,15 +25,15 @@ import scipy.special
 
 
 Mutations = collections.namedtuple(
-                        'Mutations',
-                        ['substitutions', 'deletions', 'insertions'])
+    "Mutations", ["substitutions", "deletions", "insertions"]
+)
 
 
 _MUT_REGEX = {
-    'substitution': re.compile(r'[ACGTN](?P<start>\-?\d+)[ACGTN]'),
-    'deletion': re.compile(r'del(?P<start>\-?\d+)to\-?\d+'),
-    'insertion': re.compile(r'ins(?P<start>\-?\d+)(?:len\d+|[ACGTN]+)'),
-    }
+    "substitution": re.compile(r"[ACGTN](?P<start>\-?\d+)[ACGTN\-]"),
+    "deletion": re.compile(r"del(?P<start>\-?\d+)to(?P<end>\-?\d+)"),
+    "insertion": re.compile(r"ins(?P<start>\-?\d+)(?:len\d+|[ACGTN]+)"),
+}
 """dict: Mutation regular expression matches."""
 
 
@@ -74,7 +74,7 @@ def process_mut_str(s):
         for mut_type, regex in _MUT_REGEX.items():
             match = regex.fullmatch(mut_str)
             if match:
-                start = int(match.group('start'))
+                start = int(match.group("start"))
                 mut_lists[mut_type].append((start, mut_str))
                 break
         else:
@@ -84,25 +84,26 @@ def process_mut_str(s):
         raise ValueError(f"duplicate mutation in: {s}")
 
     return Mutations(
-            substitutions=[m for _, m in sorted(mut_lists['substitution'])],
-            deletions=[m for _, m in sorted(mut_lists['deletion'])],
-            insertions=[m for _, m in sorted(mut_lists['insertion'])],
-            )
+        substitutions=[m for _, m in sorted(mut_lists["substitution"])],
+        deletions=[m for _, m in sorted(mut_lists["deletion"])],
+        insertions=[m for _, m in sorted(mut_lists["insertion"])],
+    )
 
 
-def add_mut_info_cols(df,
-                      *,
-                      mutation_col='mutations',
-                      sub_str_col=None,
-                      del_str_col=None,
-                      ins_str_col=None,
-                      indel_str_col=None,
-                      n_sub_col=None,
-                      n_del_col=None,
-                      n_ins_col=None,
-                      n_indel_col=None,
-                      overwrite_cols=False,
-                      ):
+def add_mut_info_cols(
+    df,
+    *,
+    mutation_col="mutations",
+    sub_str_col=None,
+    del_str_col=None,
+    ins_str_col=None,
+    indel_str_col=None,
+    n_sub_col=None,
+    n_del_col=None,
+    n_ins_col=None,
+    n_indel_col=None,
+    overwrite_cols=False,
+):
     """Expand information about mutations in a data frame.
 
     Parameters
@@ -155,7 +156,7 @@ def add_mut_info_cols(df,
 
     Get info on substitutions and indels:
 
-    >>> pd.set_option('max_columns', 10)
+    >>> pd.set_option('display.max_columns', 10)
     >>> add_mut_info_cols(df, sub_str_col='subs', n_sub_col='nsubs',
     ...                   indel_str_col='indels', n_indel_col='nindels')
        name            mutations     subs           indels  nsubs  nindels
@@ -173,22 +174,29 @@ def add_mut_info_cols(df,
     2  seq3  T5A ins5AAT del8to9      1      1     1        2
 
     """
+    df = df.reset_index(drop=True)
+
     if mutation_col not in df.columns:
         raise ValueError(f"`df` lacks `mutation_col` {mutation_col}")
 
     new_cols = collections.OrderedDict(
-                [(key, val)
-                 for key, val in [('substitutions_str', sub_str_col),
-                                  ('deletions_str', del_str_col),
-                                  ('insertions_str', ins_str_col),
-                                  ('indel_str', indel_str_col),
-                                  ('substitutions_n', n_sub_col),
-                                  ('deletions_n', n_del_col),
-                                  ('insertions_n', n_ins_col),
-                                  ('indel_n', n_indel_col)]
-                 if val is not None])
+        [
+            (key, val)
+            for key, val in [
+                ("substitutions_str", sub_str_col),
+                ("deletions_str", del_str_col),
+                ("insertions_str", ins_str_col),
+                ("indel_str", indel_str_col),
+                ("substitutions_n", n_sub_col),
+                ("deletions_n", n_del_col),
+                ("insertions_n", n_ins_col),
+                ("indel_n", n_indel_col),
+            ]
+            if val is not None
+        ]
+    )
     if mutation_col in set(new_cols.values()):
-        raise ValueError('`mutation_col` also name of col to add')
+        raise ValueError("`mutation_col` also name of col to add")
     already_has = set(new_cols.values()).intersection(set(df.columns))
     if already_has and not overwrite_cols:
         raise ValueError(f"`df` already has these columns: {already_has}")
@@ -197,28 +205,23 @@ def add_mut_info_cols(df,
         m = process_mut_str(s)
         returnlist = []
         for col in new_cols.keys():
-            mut_type, col_type = col.split('_')
-            if mut_type == 'indel':
+            mut_type, col_type = col.split("_")
+            if mut_type == "indel":
                 mlist = m.deletions + m.insertions
             else:
                 mlist = getattr(m, mut_type)
-            if col_type == 'str':
-                returnlist.append(' '.join(mlist))
+            if col_type == "str":
+                returnlist.append(" ".join(mlist))
             else:
-                assert col_type == 'n'
+                assert col_type == "n"
                 returnlist.append(len(mlist))
         return returnlist
 
-    mut_info_df = (pd.DataFrame(
-                        [_mut_info(m) for m in df[mutation_col].values],
-                        columns=new_cols.values())
-                   .reindex(index=df.index)
-                   )
+    mut_info_df = pd.DataFrame(
+        [_mut_info(m) for m in df[mutation_col].values], columns=new_cols.values()
+    ).reindex(index=df.index)
 
-    return (df
-            .drop(columns=new_cols.values(), errors='ignore')
-            .join(mut_info_df)
-            )
+    return df.drop(columns=new_cols.values(), errors="ignore").join(mut_info_df)
 
 
 class _LnL_error_rate:
@@ -259,26 +262,25 @@ class _LnL_error_rate:
 
     def __init__(self, df, *, n_col, u_col, count_col):
         """See main class docstring."""
-        self._df = (
-            df
-            .assign(
-                n=lambda x: x[n_col],
-                u=lambda x: x[u_col],
-                count=lambda x: x[count_col],
-                binom=lambda x: scipy.special.binom(x['n'], x['u'] - 1),
-                delta_un=lambda x: (x['n'] == x['u']).astype(int),
-                )
-            )
+        self._df = df.assign(
+            n=lambda x: x[n_col],
+            u=lambda x: x[u_col],
+            count=lambda x: x[count_col],
+            binom=lambda x: scipy.special.binom(x["n"], x["u"] - 1),
+            delta_un=lambda x: (x["n"] == x["u"]).astype(int),
+        )
 
     def lnlik(self, eps):
         """Log likelihood for error rate `eps`."""
-        return sum(self._df['count'] *
-                   numpy.log(self._df['binom'] *
-                             (1 - eps)**(self._df['n'] - self._df['u'] + 1) *
-                             eps**(self._df['u'] - 1) +
-                             self._df['delta_un'] * eps**self._df['n']
-                             )
-                   )
+        return sum(
+            self._df["count"]
+            * numpy.log(
+                self._df["binom"]
+                * (1 - eps) ** (self._df["n"] - self._df["u"] + 1)
+                * eps ** (self._df["u"] - 1)
+                + self._df["delta_un"] * eps ** self._df["n"]
+            )
+        )
 
     def neg_lnlik(self, eps):
         """Negative log likelihood for error rate `epsilon`."""
@@ -286,22 +288,23 @@ class _LnL_error_rate:
 
     def maxlik_eps(self):
         """Maximum likelihood value of error rate `epsilon`."""
-        res = scipy.optimize.minimize_scalar(self.neg_lnlik,
-                                             bounds=(1e-8, 1 - 1e-8),
-                                             method='bounded')
+        res = scipy.optimize.minimize_scalar(
+            self.neg_lnlik, bounds=(1e-8, 1 - 1e-8), method="bounded"
+        )
         if not res.success:
             raise RuntimeError(f"optimization failed:\n{res}")
         return res.x
 
 
-def empirical_accuracy(df,
-                       *,
-                       group_cols='barcode',
-                       upstream_group_cols='library',
-                       mutation_col='mutations',
-                       accuracy_col='accuracy',
-                       sort_mutations=True,
-                       ):
+def empirical_accuracy(
+    df,
+    *,
+    group_cols="barcode",
+    upstream_group_cols="library",
+    mutation_col="mutations",
+    accuracy_col="accuracy",
+    sort_mutations=True,
+):
     r"""Accuracy from number of identical sequences in a group (i.e., barcode).
 
     Note
@@ -390,7 +393,7 @@ def empirical_accuracy(df,
     >>> df = pd.DataFrame({
     ...         'barcode': ['AT', 'AT', 'TG', 'TA', 'TA', 'TA'],
     ...         'mutations': ['A1G', 'A1G', '', 'T2A C6G', 'T5C', 'C6G T2A']})
-    >>> with pd.option_context('precision', 4):
+    >>> with pd.option_context('display.precision', 4):
     ...     empirical_accuracy(df, upstream_group_cols=None)
        accuracy
     0    0.7692
@@ -398,7 +401,7 @@ def empirical_accuracy(df,
     If we do the same without sorting the mutations, we get lower values
     as 'T2A C6G' and 'C6G T2A' are then considered as different:
 
-    >>> with pd.option_context('precision', 4):
+    >>> with pd.option_context('display.precision', 4):
     ...     empirical_accuracy(df, upstream_group_cols=None,
     ...                        sort_mutations=False)
        accuracy
@@ -412,7 +415,7 @@ def empirical_accuracy(df,
     ...     'bc'  :['AT', 'AT', 'TG', 'TA', 'TA', 'TA', 'TA'],
     ...     'var' :['v1', 'v1', 'v2', 'v3', 'v4', 'v3', 'v3'],
     ...     'lib' :['s1', 's1', 's2', 's3', 's3', 's3', 's4']})
-    >>> with pd.option_context('precision', 4):
+    >>> with pd.option_context('display.precision', 4):
     ...     empirical_accuracy(df2, upstream_group_cols='lib', group_cols='bc',
     ...                        mutation_col='var', sort_mutations=False)
       lib  accuracy
@@ -420,7 +423,7 @@ def empirical_accuracy(df,
     1  s3    0.6667
 
     """
-    reserved_cols = ['_n', '_u', '_ngroups', '_dummy']
+    reserved_cols = ["_n", "_u", "_ngroups", "_dummy"]
     for col in reserved_cols:
         if col in df.columns:
             raise ValueError(f"`df` cannot have column named {col}")
@@ -432,8 +435,8 @@ def empirical_accuracy(df,
         group_cols = [group_cols]
     if (upstream_group_cols is None) or upstream_group_cols == []:
         drop_upstream_col = True
-        upstream_group_cols = '_dummy'
-        df = df.assign(**{upstream_group_cols: 'dummy'})
+        upstream_group_cols = "_dummy"
+        df = df.assign(**{upstream_group_cols: "dummy"})
     else:
         drop_upstream_col = False
     if isinstance(upstream_group_cols, str):
@@ -441,7 +444,7 @@ def empirical_accuracy(df,
 
     cols = group_cols + upstream_group_cols
     if len(set(cols)) != len(cols):
-        raise ValueError('duplicate in `group_cols` / `upstream_group_cols`')
+        raise ValueError("duplicate in `group_cols` / `upstream_group_cols`")
     if mutation_col in cols:
         raise ValueError(f"`mutation_col` {mutation_col} also grouping column")
     for col in cols + [mutation_col]:
@@ -451,54 +454,58 @@ def empirical_accuracy(df,
     result = (
         df
         # number of sequences in each group
-        .assign(_n=lambda x: x.groupby(cols)[mutation_col].transform('count'))
+        .assign(_n=lambda x: x.groupby(cols)[mutation_col].transform("count"))
         # only retain groups with >1 sequence
-        .query('_n > 1')
+        .query("_n > 1")
         # sort mutations
-        .assign(**{mutation_col: (lambda x:
-                                  x[mutation_col]
-                                  .map(lambda s: ' '.join(sorted(s.split())))
-                                  if sort_mutations else x[mutation_col]
-                                  )}
+        .assign(
+            **{
+                mutation_col: (
+                    lambda x: x[mutation_col].map(lambda s: " ".join(sorted(s.split())))
+                    if sort_mutations
+                    else x[mutation_col]
                 )
+            }
+        )
         # number of unique sequences in each group
-        .assign(_u=lambda x: (x.groupby(cols)[mutation_col]
-                              .transform('nunique'))
-                )
+        .assign(_u=lambda x: (x.groupby(cols)[mutation_col].transform("nunique")))
         # number of groups with each combination of n and u
-        .groupby(upstream_group_cols + ['_n', '_u'])
+        .groupby(upstream_group_cols + ["_n", "_u"])
         .size()
-        .rename('_ngroups')
+        .rename("_ngroups")
         .reset_index()
         # get error rate
         .groupby(upstream_group_cols)
-        .apply(lambda x: 1 - _LnL_error_rate(x,
-                                             n_col='_n',
-                                             u_col='_u',
-                                             count_col='_ngroups'
-                                             ).maxlik_eps()
-               )
+        .apply(
+            lambda x: 1
+            - _LnL_error_rate(
+                x, n_col="_n", u_col="_u", count_col="_ngroups"
+            ).maxlik_eps()
+        )
         .rename(accuracy_col)
         .reset_index()
-        )
+    )
 
     if drop_upstream_col:
         assert len(upstream_group_cols) == 1
-        result = result.drop(upstream_group_cols, axis='columns')
+        result = result.drop(upstream_group_cols, axis="columns")
 
     return result
 
 
-def simple_mutconsensus(df,
-                        *,
-                        group_cols=('library', 'barcode'),
-                        mutation_col='mutations',
-                        max_sub_diffs=1,
-                        max_indel_diffs=2,
-                        max_minor_sub_frac=0.1,
-                        max_minor_indel_frac=0.25,
-                        support_col='variant_call_support',
-                        ):
+def simple_mutconsensus(
+    df,
+    *,
+    group_cols=("library", "barcode"),
+    mutation_col="mutations",
+    max_sub_diffs=1,
+    max_indel_diffs=2,
+    max_minor_sub_frac=0.1,
+    max_minor_indel_frac=0.25,
+    max_minor_greater_or_equal=False,
+    min_support=1,
+    support_col="variant_call_support",
+):
     """Get simple consensus of mutations with group (i.e., barcode).
 
     Parameters
@@ -509,19 +516,26 @@ def simple_mutconsensus(df,
         Group all sequences that share values in these column(s) of `df`.
     mutation_col : str
         Column in `df` with mutations in form that can be processed by
-        :func:`processe_mut_str`.
-    max_sub_diffs : int
+        :func:`process_mut_str`.
+    max_sub_diffs : int or None
         Drop groups where any variant differs from all other variants
         by more than this many substitution (point mutation) differences.
-    max_indel_diffs : int
+        Set to ``None`` if no limit.
+    max_indel_diffs : int or None
         Drop groups where any variant differs from all other variants
-        by more than this many indel differences.
+        by more than this many indel differences. Set to ``None`` if no limit.
     max_minor_sub_frac : float
-        Drop groups with a minor (non-consensus) substitution in more than the
+        Drop groups with a minor (non-consensus) substitution in > the
         **ceiling** of this fraction times the number of sequences in group.
     max_minor_indel_frac : float
-        Drop groups with a minor (non-consensus) indel in more than the
+        Drop groups with a minor (non-consensus) indel in > the
         **ceiling** of this fraction times the number of sequences in group.
+    max_minor_greater_or_equal : bool
+        For ``max_minor_sub_frac`` and ``max_minor_indel_frac``, use >= rather
+        than >. This is may help if you want to be conservative in
+        avoiding bad consensuses. But don't reject if zero differences.
+    min_support : int
+        Require at least this many supporting sequences to call consensus.
     support_col : str
           Name of column in returned `consensus` data frame with number
           of sequences supporting the consensus call.
@@ -564,6 +578,8 @@ def simple_mutconsensus(df,
     ...        lib_1,AG,A2C del5to7
     ...        lib_1,AG,A2C
     ...        lib_1,TA,G3A ins4len3
+    ...        lib_1,TA,G3A ins4len3
+    ...        lib_1,TA,G3A ins4len3
     ...        lib_2,TA,C5A T-6C
     ...        lib_2,TA,ins5len1 T-6C
     ...        lib_2,TA,T-6C
@@ -592,7 +608,7 @@ def simple_mutconsensus(df,
     >>> consensus
       library barcode     mutations  variant_call_support
     0   lib_1      AG           A2C                     2
-    1   lib_1      TA  G3A ins4len3                     1
+    1   lib_1      TA  G3A ins4len3                     3
     2   lib_2      GG                                   2
     3   lib_2      TA          T-6C                     4
     >>> dropped
@@ -608,7 +624,53 @@ def simple_mutconsensus(df,
     >>> lib1_consensus
       barcode     mutations  variant_call_support
     0      AG           A2C                     2
-    1      TA  G3A ins4len3                     1
+    1      TA  G3A ins4len3                     3
+
+    Set ``max_sub_diffs`` to None:
+
+    >>> consensus, dropped = simple_mutconsensus(df, max_sub_diffs=None)
+    >>> consensus
+      library barcode     mutations  variant_call_support
+    0   lib_1      AG           A2C                     2
+    1   lib_1      TA  G3A ins4len3                     3
+    2   lib_2      GG                                   2
+    3   lib_2      TA          T-6C                     4
+    4   lib_2      TG                                   2
+    >>> dropped
+      library barcode              drop_reason  nseqs
+    0   lib_2      AA  minor subs too frequent      4
+    1   lib_3      AA    indels diff too large      2
+
+    Set ``max_minor_greater_or_equal`` to True:
+
+    >>> consensus, dropped = simple_mutconsensus(
+    ...         df, max_minor_greater_or_equal=True)
+    >>> consensus
+      library barcode     mutations  variant_call_support
+    0   lib_1      TA  G3A ins4len3                     3
+    >>> dropped
+      library barcode                drop_reason  nseqs
+    0   lib_1      AG  minor indels too frequent      2
+    1   lib_2      AA    minor subs too frequent      4
+    2   lib_2      GG    minor subs too frequent      2
+    3   lib_2      TA    minor subs too frequent      4
+    4   lib_2      TG        subs diff too large      2
+    5   lib_3      AA      indels diff too large      2
+
+    Use ``min_support``:
+
+    >>> consensus, dropped = simple_mutconsensus(df, min_support=3)
+    >>> consensus
+      library barcode     mutations  variant_call_support
+    0   lib_1      TA  G3A ins4len3                     3
+    1   lib_2      TA          T-6C                     4
+    >>> dropped
+      library barcode              drop_reason  nseqs
+    0   lib_1      AG        too few sequences      2
+    1   lib_2      AA  minor subs too frequent      4
+    2   lib_2      GG        too few sequences      2
+    3   lib_2      TG        too few sequences      2
+    4   lib_3      AA        too few sequences      2
 
     """
     if isinstance(group_cols, str):
@@ -624,74 +686,89 @@ def simple_mutconsensus(df,
 
     df = df[group_cols + [mutation_col]]
     if df.isnull().values.any():
-        raise ValueError('`df` contains `na` (null) entries. It is possible '
-                         'that you read a CSV without using `na_filter=False` '
-                         'so that empty mutation strings were read as `na` '
-                         'rather than empty strings.')
+        raise ValueError(
+            "`df` contains `na` (null) entries. It is possible "
+            "that you read a CSV without using `na_filter=False` "
+            "so that empty mutation strings were read as `na` "
+            "rather than empty strings."
+        )
 
     dropped = []
     consensus = []
     for g, g_df in df.groupby(group_cols, observed=True)[mutation_col]:
 
+        if len(group_cols) == 1:
+            g = [g]
+
         nseqs = len(g_df)
         half_nseqs = 0.5 * nseqs
         assert nseqs > 0
 
+        if nseqs < min_support:
+            dropped.append((*g, "too few sequences", nseqs))
+            continue
+
         mutations = [process_mut_str(s) for s in g_df.values]
 
         drop_reason = None
         g_consensus = []
 
-        mutlists = {'subs': [m.substitutions for m in mutations],
-                    'indels': [m.deletions + m.insertions for m in mutations],
-                    }
+        mutlists = {
+            "subs": [m.substitutions for m in mutations],
+            "indels": [m.deletions + m.insertions for m in mutations],
+        }
 
         for mtype, maxd, max_frac in [
-                        ('subs', max_sub_diffs, max_minor_sub_frac),
-                        ('indels', max_indel_diffs, max_minor_indel_frac),
-                        ]:
-
-            # are max_sub_diffs and max_indel_diffs satisfied?
-            ndiffs_by_seq = collections.defaultdict(list)
-            for (i1, m1set), (i2, m2set) in itertools.combinations(
-                        enumerate(mutlists[mtype]), 2):
-                ndiffs = len(numpy.setxor1d(m1set, m2set, assume_unique=True))
-                ndiffs_by_seq[i1].append(ndiffs)
-                ndiffs_by_seq[i2].append(ndiffs)
-            if any(min(ndifflist) > maxd for ndifflist
-                   in ndiffs_by_seq.values()):
-                drop_reason = f"{mtype} diff too large"
-                break
+            ("subs", max_sub_diffs, max_minor_sub_frac),
+            ("indels", max_indel_diffs, max_minor_indel_frac),
+        ]:
+            if maxd is not None:
+                # is max_sub_diffs or max_indel_diffs satisfied?
+                ndiffs_by_seq = collections.defaultdict(list)
+                for (i1, m1set), (i2, m2set) in itertools.combinations(
+                    enumerate(mutlists[mtype]), 2
+                ):
+                    ndiffs = len(numpy.setxor1d(m1set, m2set, assume_unique=True))
+                    ndiffs_by_seq[i1].append(ndiffs)
+                    ndiffs_by_seq[i2].append(ndiffs)
+                if any(min(ndifflist) > maxd for ndifflist in ndiffs_by_seq.values()):
+                    drop_reason = f"{mtype} diff too large"
+                    break
 
             # see if max_minor_mut_frac is satisfied
             max_muts = math.ceil(max_frac * nseqs)
             nseqs_minus_max_muts = nseqs - max_muts
-            counts = collections.Counter(itertools.chain.from_iterable(
-                            mutlists[mtype]))
-            if any(max_muts < count < nseqs_minus_max_muts for count
-                   in counts.values()):
+            counts = collections.Counter(itertools.chain.from_iterable(mutlists[mtype]))
+            if max_minor_greater_or_equal:
+                if any(
+                    (max_muts <= count <= nseqs_minus_max_muts) and (count != nseqs)
+                    for count in counts.values()
+                ):
+                    drop_reason = f"minor {mtype} too frequent"
+                    break
+            elif any(
+                max_muts < count < nseqs_minus_max_muts for count in counts.values()
+            ):
                 drop_reason = f"minor {mtype} too frequent"
                 break
 
             # consensus for mutation type
             g_consensus += [m for m, c in counts.items() if c > half_nseqs]
 
-        if len(group_cols) == 1:
-            g = [g]
-
         if drop_reason is not None:
             dropped.append((*g, drop_reason, nseqs))
         else:
-            consensus.append((*g, ' '.join(g_consensus), nseqs))
+            consensus.append((*g, " ".join(g_consensus), nseqs))
 
-    consensus = pd.DataFrame(consensus,
-                             columns=group_cols + [mutation_col, support_col])
-    dropped = pd.DataFrame(dropped,
-                           columns=group_cols + ['drop_reason', 'nseqs'])
+    consensus = pd.DataFrame(
+        consensus, columns=group_cols + [mutation_col, support_col]
+    )
+    dropped = pd.DataFrame(dropped, columns=group_cols + ["drop_reason", "nseqs"])
 
     return consensus, dropped
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     import doctest
+
     doctest.testmod()
