@@ -8,8 +8,16 @@ Defines constants used by package.
 """
 
 
-CBPALETTE = ('#999999', '#E69F00', '#56B4E9', '#009E73',
-             '#F0E442', '#0072B2', '#D55E00', '#CC79A7')
+CBPALETTE = (
+    "#999999",
+    "#E69F00",
+    "#56B4E9",
+    "#009E73",
+    "#F0E442",
+    "#0072B2",
+    "#D55E00",
+    "#CC79A7",
+)
 """tuple: Color-blind safe palette.
 
 From http://bconnelly.net/2013/10/creating-colorblind-friendly-figures/
